

set -ex



pip check
if [ ! -f $CONDA_PREFIX/etc/conda/activate.d/anaconda-ident_activate.sh ]; then exit 1; fi
if [ ! -f $CONDA_PREFIX/bin/.anaconda-ident-pre-unlink.sh ]; then exit 1; fi
if [ ! -f $CONDA_PREFIX/bin/.anaconda-ident-post-link.sh ]; then exit 1; fi
export ANACONDA_IDENT_DEBUG=1
export PYTHONUNBUFFERED=1
conda create -n testchild1 --yes
conda create -n testchild2 --yes
conda info
conda info --envs
anaconda-ident --expect
anaconda-keymgr --help
python tests/test_config.py
exit 0
